@@ -3,13 +3,19 @@ use std::{env, fs, io};
 
 pub fn download(name: impl AsRef<str>, url: impl AsRef<str>) -> io::Result<PathBuf> {
     let resp = ureq::get(url.as_ref()).call();
+    let download_dir = Path::new(&env::var("OUT_DIR").unwrap()).join("download");
+    fs::create_dir_all(&download_dir).unwrap();
+    let output_path = download_dir.join(name.as_ref());
+
+    if fs::File::open(&output_path).is_ok() {
+        return Ok(output_path);
+    }
+
     match resp {
         Ok(resp) => {
             println!("ok");
             let mut reader = resp.into_reader();
-            let download_dir = Path::new(&env::var("OUT_DIR").unwrap()).join("download");
-            fs::create_dir_all(&download_dir).unwrap();
-            let output_path = download_dir.join(name.as_ref());
+            
             let mut output_file = fs::File::create(&output_path)?;
             io::copy(&mut reader, &mut output_file)?;
             Ok(output_path)
